using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using System.Reflection;

namespace testcases.CWE398_Code_Quality
{
	class Program {

		static void Main(string[] args) {
			
			if(args.Any()) {
			
				if(args[0].Equals("-h", StringComparison.OrdinalIgnoreCase) || 
                   args[0].Equals("--help", StringComparison.OrdinalIgnoreCase)) {
			
					Console.WriteLine("To use this main, you can either run the program with no " +
					"command line arguments to run all test cases or you can specify one or more classes to test");
					System.Environment.Exit(1);
				}
				
				/* User supplied some class names on the command line, just use those with introspection
				 *
				 * string classNames[] = { "CWE481_Assigning_instead_of_Comparing__boolean_01",
				 *		"CWE476_Null_Pointer_Dereference__getProperty_01" };
				 * could read class names from command line or use
				 * http://sadun-util.sourceforge.net/api/org/sadun/util/
				 * ClassPackageExplorer.html
				 */

				foreach (string className in args) {

					try {
						
						/* String classNameWithPackage = "testcases." + className; */
						
						/* Console.WriteLine("classNameWithPackage = " + classNameWithPackage); */

						Type myClass = Type.GetType(className);
						object myObject = Activator.CreateInstance(myClass);
						myClass.InvokeMember("runTest", 
							BindingFlags.InvokeMethod | BindingFlags.Instance | BindingFlags.Public, 
							null, 
							myObject, 
							new object[] { className });

					} catch (Exception ex) {

						Console.WriteLine("Could not run test for class " + className);
						Console.WriteLine(ex.StackTrace);

					}
					
					Console.WriteLine(""); /* leave a blank line between classes */

				}

			} else {
			
				/* No command line args were used, we want to run every testcase */
				
				/* needed to separate these calls into other methods because
				   we were running into the size limit Java has for a single method */
				RunTestCWE1();
				RunTestCWE2();
				RunTestCWE3();
				RunTestCWE4();
				RunTestCWE5();
				RunTestCWE6();
				RunTestCWE7();
				RunTestCWE8();
				RunTestCWE9();
			}			
		}

	private static void RunTestCWE1() {
	/* BEGIN-AUTOGENERATED-CSHARP-TESTS-1 */

			/* END-AUTOGENERATED-CSHARP-TESTS-1 */
	}

	private static void RunTestCWE2() {
	/* BEGIN-AUTOGENERATED-CSHARP-TESTS-2 */

			/* END-AUTOGENERATED-CSHARP-TESTS-2 */
	}

	private static void RunTestCWE3() {
	/* BEGIN-AUTOGENERATED-CSHARP-TESTS-3 */
			(new CWE398_Code_Quality__empty_block_01()).RunTest("CWE398_Code_Quality__empty_block_01");
			(new CWE398_Code_Quality__empty_block_02()).RunTest("CWE398_Code_Quality__empty_block_02");
			(new CWE398_Code_Quality__empty_block_03()).RunTest("CWE398_Code_Quality__empty_block_03");
			(new CWE398_Code_Quality__empty_block_04()).RunTest("CWE398_Code_Quality__empty_block_04");
			(new CWE398_Code_Quality__empty_block_05()).RunTest("CWE398_Code_Quality__empty_block_05");
			(new CWE398_Code_Quality__empty_block_06()).RunTest("CWE398_Code_Quality__empty_block_06");
			(new CWE398_Code_Quality__empty_block_07()).RunTest("CWE398_Code_Quality__empty_block_07");
			(new CWE398_Code_Quality__empty_block_08()).RunTest("CWE398_Code_Quality__empty_block_08");
			(new CWE398_Code_Quality__empty_block_09()).RunTest("CWE398_Code_Quality__empty_block_09");
			(new CWE398_Code_Quality__empty_block_10()).RunTest("CWE398_Code_Quality__empty_block_10");
			(new CWE398_Code_Quality__empty_block_11()).RunTest("CWE398_Code_Quality__empty_block_11");
			(new CWE398_Code_Quality__empty_block_12()).RunTest("CWE398_Code_Quality__empty_block_12");
			(new CWE398_Code_Quality__empty_block_13()).RunTest("CWE398_Code_Quality__empty_block_13");
			(new CWE398_Code_Quality__empty_block_14()).RunTest("CWE398_Code_Quality__empty_block_14");
			(new CWE398_Code_Quality__empty_block_15()).RunTest("CWE398_Code_Quality__empty_block_15");
			(new CWE398_Code_Quality__empty_block_16()).RunTest("CWE398_Code_Quality__empty_block_16");
			(new CWE398_Code_Quality__empty_block_17()).RunTest("CWE398_Code_Quality__empty_block_17");
			(new CWE398_Code_Quality__empty_case_01()).RunTest("CWE398_Code_Quality__empty_case_01");
			(new CWE398_Code_Quality__empty_case_02()).RunTest("CWE398_Code_Quality__empty_case_02");
			(new CWE398_Code_Quality__empty_case_03()).RunTest("CWE398_Code_Quality__empty_case_03");
			(new CWE398_Code_Quality__empty_case_04()).RunTest("CWE398_Code_Quality__empty_case_04");
			(new CWE398_Code_Quality__empty_case_05()).RunTest("CWE398_Code_Quality__empty_case_05");
			(new CWE398_Code_Quality__empty_case_06()).RunTest("CWE398_Code_Quality__empty_case_06");
			(new CWE398_Code_Quality__empty_case_07()).RunTest("CWE398_Code_Quality__empty_case_07");
			(new CWE398_Code_Quality__empty_case_08()).RunTest("CWE398_Code_Quality__empty_case_08");
			(new CWE398_Code_Quality__empty_case_09()).RunTest("CWE398_Code_Quality__empty_case_09");
			(new CWE398_Code_Quality__empty_case_10()).RunTest("CWE398_Code_Quality__empty_case_10");
			(new CWE398_Code_Quality__empty_case_11()).RunTest("CWE398_Code_Quality__empty_case_11");
			(new CWE398_Code_Quality__empty_case_12()).RunTest("CWE398_Code_Quality__empty_case_12");
			(new CWE398_Code_Quality__empty_case_13()).RunTest("CWE398_Code_Quality__empty_case_13");
			(new CWE398_Code_Quality__empty_case_14()).RunTest("CWE398_Code_Quality__empty_case_14");
			(new CWE398_Code_Quality__empty_case_15()).RunTest("CWE398_Code_Quality__empty_case_15");
			(new CWE398_Code_Quality__empty_case_16()).RunTest("CWE398_Code_Quality__empty_case_16");
			(new CWE398_Code_Quality__empty_case_17()).RunTest("CWE398_Code_Quality__empty_case_17");
			(new CWE398_Code_Quality__empty_else_01()).RunTest("CWE398_Code_Quality__empty_else_01");
			(new CWE398_Code_Quality__empty_else_02()).RunTest("CWE398_Code_Quality__empty_else_02");
			(new CWE398_Code_Quality__empty_else_03()).RunTest("CWE398_Code_Quality__empty_else_03");
			(new CWE398_Code_Quality__empty_else_04()).RunTest("CWE398_Code_Quality__empty_else_04");
			(new CWE398_Code_Quality__empty_else_05()).RunTest("CWE398_Code_Quality__empty_else_05");
			(new CWE398_Code_Quality__empty_else_06()).RunTest("CWE398_Code_Quality__empty_else_06");
			(new CWE398_Code_Quality__empty_else_07()).RunTest("CWE398_Code_Quality__empty_else_07");
			(new CWE398_Code_Quality__empty_else_08()).RunTest("CWE398_Code_Quality__empty_else_08");
			(new CWE398_Code_Quality__empty_else_09()).RunTest("CWE398_Code_Quality__empty_else_09");
			(new CWE398_Code_Quality__empty_else_10()).RunTest("CWE398_Code_Quality__empty_else_10");
			(new CWE398_Code_Quality__empty_else_11()).RunTest("CWE398_Code_Quality__empty_else_11");
			(new CWE398_Code_Quality__empty_else_12()).RunTest("CWE398_Code_Quality__empty_else_12");
			(new CWE398_Code_Quality__empty_else_13()).RunTest("CWE398_Code_Quality__empty_else_13");
			(new CWE398_Code_Quality__empty_else_14()).RunTest("CWE398_Code_Quality__empty_else_14");
			(new CWE398_Code_Quality__empty_else_15()).RunTest("CWE398_Code_Quality__empty_else_15");
			(new CWE398_Code_Quality__empty_else_16()).RunTest("CWE398_Code_Quality__empty_else_16");
			(new CWE398_Code_Quality__empty_else_17()).RunTest("CWE398_Code_Quality__empty_else_17");
			(new CWE398_Code_Quality__empty_for_01()).RunTest("CWE398_Code_Quality__empty_for_01");
			(new CWE398_Code_Quality__empty_for_02()).RunTest("CWE398_Code_Quality__empty_for_02");
			(new CWE398_Code_Quality__empty_for_03()).RunTest("CWE398_Code_Quality__empty_for_03");
			(new CWE398_Code_Quality__empty_for_04()).RunTest("CWE398_Code_Quality__empty_for_04");
			(new CWE398_Code_Quality__empty_for_05()).RunTest("CWE398_Code_Quality__empty_for_05");
			(new CWE398_Code_Quality__empty_for_06()).RunTest("CWE398_Code_Quality__empty_for_06");
			(new CWE398_Code_Quality__empty_for_07()).RunTest("CWE398_Code_Quality__empty_for_07");
			(new CWE398_Code_Quality__empty_for_08()).RunTest("CWE398_Code_Quality__empty_for_08");
			(new CWE398_Code_Quality__empty_for_09()).RunTest("CWE398_Code_Quality__empty_for_09");
			(new CWE398_Code_Quality__empty_for_10()).RunTest("CWE398_Code_Quality__empty_for_10");
			(new CWE398_Code_Quality__empty_for_11()).RunTest("CWE398_Code_Quality__empty_for_11");
			(new CWE398_Code_Quality__empty_for_12()).RunTest("CWE398_Code_Quality__empty_for_12");
			(new CWE398_Code_Quality__empty_for_13()).RunTest("CWE398_Code_Quality__empty_for_13");
			(new CWE398_Code_Quality__empty_for_14()).RunTest("CWE398_Code_Quality__empty_for_14");
			(new CWE398_Code_Quality__empty_for_15()).RunTest("CWE398_Code_Quality__empty_for_15");
			(new CWE398_Code_Quality__empty_for_16()).RunTest("CWE398_Code_Quality__empty_for_16");
			(new CWE398_Code_Quality__empty_for_17()).RunTest("CWE398_Code_Quality__empty_for_17");
			(new CWE398_Code_Quality__empty_function_01()).RunTest("CWE398_Code_Quality__empty_function_01");
			(new CWE398_Code_Quality__empty_if_01()).RunTest("CWE398_Code_Quality__empty_if_01");
			(new CWE398_Code_Quality__empty_if_02()).RunTest("CWE398_Code_Quality__empty_if_02");
			(new CWE398_Code_Quality__empty_if_03()).RunTest("CWE398_Code_Quality__empty_if_03");
			(new CWE398_Code_Quality__empty_if_04()).RunTest("CWE398_Code_Quality__empty_if_04");
			(new CWE398_Code_Quality__empty_if_05()).RunTest("CWE398_Code_Quality__empty_if_05");
			(new CWE398_Code_Quality__empty_if_06()).RunTest("CWE398_Code_Quality__empty_if_06");
			(new CWE398_Code_Quality__empty_if_07()).RunTest("CWE398_Code_Quality__empty_if_07");
			(new CWE398_Code_Quality__empty_if_08()).RunTest("CWE398_Code_Quality__empty_if_08");
			(new CWE398_Code_Quality__empty_if_09()).RunTest("CWE398_Code_Quality__empty_if_09");
			(new CWE398_Code_Quality__empty_if_10()).RunTest("CWE398_Code_Quality__empty_if_10");
			(new CWE398_Code_Quality__empty_if_11()).RunTest("CWE398_Code_Quality__empty_if_11");
			(new CWE398_Code_Quality__empty_if_12()).RunTest("CWE398_Code_Quality__empty_if_12");
			(new CWE398_Code_Quality__empty_if_13()).RunTest("CWE398_Code_Quality__empty_if_13");
			(new CWE398_Code_Quality__empty_if_14()).RunTest("CWE398_Code_Quality__empty_if_14");
			(new CWE398_Code_Quality__empty_if_15()).RunTest("CWE398_Code_Quality__empty_if_15");
			(new CWE398_Code_Quality__empty_if_16()).RunTest("CWE398_Code_Quality__empty_if_16");
			(new CWE398_Code_Quality__empty_if_17()).RunTest("CWE398_Code_Quality__empty_if_17");
			(new CWE398_Code_Quality__empty_while_01()).RunTest("CWE398_Code_Quality__empty_while_01");
			(new CWE398_Code_Quality__empty_while_02()).RunTest("CWE398_Code_Quality__empty_while_02");
			(new CWE398_Code_Quality__empty_while_03()).RunTest("CWE398_Code_Quality__empty_while_03");
			(new CWE398_Code_Quality__empty_while_04()).RunTest("CWE398_Code_Quality__empty_while_04");
			(new CWE398_Code_Quality__empty_while_05()).RunTest("CWE398_Code_Quality__empty_while_05");
			(new CWE398_Code_Quality__empty_while_06()).RunTest("CWE398_Code_Quality__empty_while_06");
			(new CWE398_Code_Quality__empty_while_07()).RunTest("CWE398_Code_Quality__empty_while_07");
			(new CWE398_Code_Quality__empty_while_08()).RunTest("CWE398_Code_Quality__empty_while_08");
			(new CWE398_Code_Quality__empty_while_09()).RunTest("CWE398_Code_Quality__empty_while_09");
			(new CWE398_Code_Quality__empty_while_10()).RunTest("CWE398_Code_Quality__empty_while_10");
			(new CWE398_Code_Quality__empty_while_11()).RunTest("CWE398_Code_Quality__empty_while_11");
			(new CWE398_Code_Quality__empty_while_12()).RunTest("CWE398_Code_Quality__empty_while_12");
			(new CWE398_Code_Quality__empty_while_13()).RunTest("CWE398_Code_Quality__empty_while_13");
			(new CWE398_Code_Quality__empty_while_14()).RunTest("CWE398_Code_Quality__empty_while_14");
			(new CWE398_Code_Quality__empty_while_15()).RunTest("CWE398_Code_Quality__empty_while_15");
			(new CWE398_Code_Quality__empty_while_16()).RunTest("CWE398_Code_Quality__empty_while_16");
			(new CWE398_Code_Quality__empty_while_17()).RunTest("CWE398_Code_Quality__empty_while_17");
			(new CWE398_Code_Quality__equals_01()).RunTest("CWE398_Code_Quality__equals_01");
			(new CWE398_Code_Quality__equals_02()).RunTest("CWE398_Code_Quality__equals_02");
			(new CWE398_Code_Quality__equals_03()).RunTest("CWE398_Code_Quality__equals_03");
			(new CWE398_Code_Quality__equals_04()).RunTest("CWE398_Code_Quality__equals_04");
			(new CWE398_Code_Quality__equals_05()).RunTest("CWE398_Code_Quality__equals_05");
			(new CWE398_Code_Quality__equals_06()).RunTest("CWE398_Code_Quality__equals_06");
			(new CWE398_Code_Quality__equals_07()).RunTest("CWE398_Code_Quality__equals_07");
			(new CWE398_Code_Quality__equals_08()).RunTest("CWE398_Code_Quality__equals_08");
			(new CWE398_Code_Quality__equals_09()).RunTest("CWE398_Code_Quality__equals_09");
			(new CWE398_Code_Quality__equals_10()).RunTest("CWE398_Code_Quality__equals_10");
			(new CWE398_Code_Quality__equals_11()).RunTest("CWE398_Code_Quality__equals_11");
			(new CWE398_Code_Quality__equals_12()).RunTest("CWE398_Code_Quality__equals_12");
			(new CWE398_Code_Quality__equals_13()).RunTest("CWE398_Code_Quality__equals_13");
			(new CWE398_Code_Quality__equals_14()).RunTest("CWE398_Code_Quality__equals_14");
			(new CWE398_Code_Quality__equals_15()).RunTest("CWE398_Code_Quality__equals_15");
			(new CWE398_Code_Quality__equals_16()).RunTest("CWE398_Code_Quality__equals_16");
			(new CWE398_Code_Quality__equals_17()).RunTest("CWE398_Code_Quality__equals_17");
			(new CWE398_Code_Quality__semicolon_01()).RunTest("CWE398_Code_Quality__semicolon_01");
			(new CWE398_Code_Quality__semicolon_02()).RunTest("CWE398_Code_Quality__semicolon_02");
			(new CWE398_Code_Quality__semicolon_03()).RunTest("CWE398_Code_Quality__semicolon_03");
			(new CWE398_Code_Quality__semicolon_04()).RunTest("CWE398_Code_Quality__semicolon_04");
			(new CWE398_Code_Quality__semicolon_05()).RunTest("CWE398_Code_Quality__semicolon_05");
			(new CWE398_Code_Quality__semicolon_06()).RunTest("CWE398_Code_Quality__semicolon_06");
			(new CWE398_Code_Quality__semicolon_07()).RunTest("CWE398_Code_Quality__semicolon_07");
			(new CWE398_Code_Quality__semicolon_08()).RunTest("CWE398_Code_Quality__semicolon_08");
			(new CWE398_Code_Quality__semicolon_09()).RunTest("CWE398_Code_Quality__semicolon_09");
			(new CWE398_Code_Quality__semicolon_10()).RunTest("CWE398_Code_Quality__semicolon_10");
			(new CWE398_Code_Quality__semicolon_11()).RunTest("CWE398_Code_Quality__semicolon_11");
			(new CWE398_Code_Quality__semicolon_12()).RunTest("CWE398_Code_Quality__semicolon_12");
			(new CWE398_Code_Quality__semicolon_13()).RunTest("CWE398_Code_Quality__semicolon_13");
			(new CWE398_Code_Quality__semicolon_14()).RunTest("CWE398_Code_Quality__semicolon_14");
			(new CWE398_Code_Quality__semicolon_15()).RunTest("CWE398_Code_Quality__semicolon_15");
			(new CWE398_Code_Quality__semicolon_16()).RunTest("CWE398_Code_Quality__semicolon_16");
			(new CWE398_Code_Quality__semicolon_17()).RunTest("CWE398_Code_Quality__semicolon_17");
			/* END-AUTOGENERATED-CSHARP-TESTS-3 */
	}

	private static void RunTestCWE4() {
	/* BEGIN-AUTOGENERATED-CSHARP-TESTS-4 */

			/* END-AUTOGENERATED-CSHARP-TESTS-4 */
	}

	private static void RunTestCWE5() {
	/* BEGIN-AUTOGENERATED-CSHARP-TESTS-5 */

			/* END-AUTOGENERATED-CSHARP-TESTS-5 */
	}

	private static void RunTestCWE6() {
	/* BEGIN-AUTOGENERATED-CSHARP-TESTS-6 */

			/* END-AUTOGENERATED-CSHARP-TESTS-6 */
	}

	private static void RunTestCWE7() {
	/* BEGIN-AUTOGENERATED-CSHARP-TESTS-7 */

			/* END-AUTOGENERATED-CSHARP-TESTS-7 */
	}

	private static void RunTestCWE8() {
	/* BEGIN-AUTOGENERATED-CSHARP-TESTS-8 */

			/* END-AUTOGENERATED-CSHARP-TESTS-8 */
	}

	private static void RunTestCWE9() {
	/* BEGIN-AUTOGENERATED-CSHARP-TESTS-9 */

			/* END-AUTOGENERATED-CSHARP-TESTS-9 */
	}
}
}