/* TEMPLATE GENERATED TESTCASE FILE
Filename: CWE89_SQL_Injection__Web_Listen_tcp_ExecuteScalar_61a.cs
Label Definition File: CWE89_SQL_Injection__Web.label.xml
Template File: sources-sinks-61a.tmpl.cs
*/
/*
 * @description
 * CWE: 89 SQL Injection
 * BadSource: Listen_tcp Read data using a listening tcp connection
 * GoodSource: A hardcoded string
 * Sinks: ExecuteScalar
 *    GoodSink: Use prepared statement and ExecuteScalar() (properly)
 *    BadSink : data concatenated into SQL statement used in ExecuteScalar(), which could result in SQL Injection
 * Flow Variant: 61 Data flow: data returned from one method to another in different classes in the same package
 *
 * */

using TestCaseSupport;
using System;

using System.Data.SqlClient;
using System.Data;
using System.Web;

namespace testcases.CWE89_SQL_Injection
{
class CWE89_SQL_Injection__Web_Listen_tcp_ExecuteScalar_61a : AbstractTestCaseWeb
{
#if (!OMITBAD)
    public override void Bad(HttpRequest req, HttpResponse resp)
    {
        string data = CWE89_SQL_Injection__Web_Listen_tcp_ExecuteScalar_61b.BadSource(req, resp);
        try
        {
            using (SqlConnection dbConnection = IO.GetDBConnection())
            {
                dbConnection.Open();
                using (SqlCommand badSqlCommand = new SqlCommand(null, dbConnection))
                {
                    /* POTENTIAL FLAW: data concatenated into SQL statement used in ExecuteScalar(), which could result in SQL Injection */
                    badSqlCommand.CommandText = "select * from users where name='" +data+"'";
                    object firstCol = badSqlCommand.ExecuteScalar();
                    if (firstCol != null)
                    {
                        IO.WriteLine(firstCol.ToString()); /* Use ResultSet in some way */
                    }
                }
            }
        }
        catch (SqlException exceptSql)
        {
            IO.Logger.Log(NLog.LogLevel.Warn, "Error getting database connection", exceptSql);
        }
    }
#endif //omitbad
#if (!OMITGOOD)
    public override void Good(HttpRequest req, HttpResponse resp)
    {
        GoodG2B(req, resp);
        GoodB2G(req, resp);
    }

    /* goodG2B() - use goodsource and badsink */
    private static void GoodG2B(HttpRequest req, HttpResponse resp)
    {
        string data = CWE89_SQL_Injection__Web_Listen_tcp_ExecuteScalar_61b.GoodG2BSource(req, resp);
        try
        {
            using (SqlConnection dbConnection = IO.GetDBConnection())
            {
                dbConnection.Open();
                using (SqlCommand badSqlCommand = new SqlCommand(null, dbConnection))
                {
                    /* POTENTIAL FLAW: data concatenated into SQL statement used in ExecuteScalar(), which could result in SQL Injection */
                    badSqlCommand.CommandText = "select * from users where name='" +data+"'";
                    object firstCol = badSqlCommand.ExecuteScalar();
                    if (firstCol != null)
                    {
                        IO.WriteLine(firstCol.ToString()); /* Use ResultSet in some way */
                    }
                }
            }
        }
        catch (SqlException exceptSql)
        {
            IO.Logger.Log(NLog.LogLevel.Warn, "Error getting database connection", exceptSql);
        }
    }

    /* goodB2G() - use badsource and goodsink */
    private static void GoodB2G(HttpRequest req, HttpResponse resp)
    {
        string data = CWE89_SQL_Injection__Web_Listen_tcp_ExecuteScalar_61b.GoodB2GSource(req, resp);
        try
        {
            using (SqlConnection dbConnection = IO.GetDBConnection())
            {
                dbConnection.Open();
                using (SqlCommand goodSqlCommand = new SqlCommand(null, dbConnection))
                {
                    /* FIX: Use prepared statement and concatenate ExecuteScalar() (properly) */
                    SqlParameter nameParam = new SqlParameter("@name", SqlDbType.VarChar, 100);
                    nameParam.Value = data;
                    goodSqlCommand.CommandText += "select * from users where name=@name";
                    goodSqlCommand.Prepare();
                    object firstCol = goodSqlCommand.ExecuteScalar();
                    if (firstCol != null)
                    {
                        IO.WriteLine(firstCol.ToString()); /* Use ResultSet in some way */
                    }
                }
            }
        }
        catch (SqlException exceptSql)
        {
            IO.Logger.Log(NLog.LogLevel.Warn, "Error getting database connection", exceptSql);
        }
    }
#endif //omitgood
}
}
